#!/usr/bin/env python
# c: 21.09.2008
import sys
sys.path.append( '.' )

import os
import numpy as nm
#from sfepy.fem import MeshIO

##########
filename_mesh = 'mesh_circ21.mesh'
##########
t0 = 0.0
t1 = 0.5
n_step = 11
##########
specific_heat = 1.2
##########

cwd = os.path.split( os.path.join( os.getcwd(), __file__ ) )[0]
filename_mesh = os.path.join( cwd, 'meshes', filename_mesh )

#dim = MeshIO.any_from_filename( filename_mesh ).read_dimension()
dim = 2
geom = {3 : '3_4', 2 : '2_3'}[dim]

x_left, x_right = -0.4999,  0.4999
    
regions = {
    'Omega' : ('all', {}),
    'Omega1' : ('elements of group 1', {}),
    'Omega2' : ('elements of group 2', {}),
    'Omega2_Surface': ('r.Omega1 *n r.Omega2', {'can_cells' : False}),
    'Left' : ('nodes in (x < %f)' % x_left, {}),
    'Right' : ('nodes in (x > %f)' % x_right, {}),
}

material_1 = {
    'name' : 'm',
    'mode' : 'here',
    'region' : 'Omega',

    'thermal_conductivity' : 2.0,
    'electric_conductivity' : 1.5,
}

field_0 = {
    'name' : 'temperature',
    'dim' : (1,1),
    'domain' : 'Omega',
    'bases' : {'Omega' : '%s_P1' % geom}
}

field_2 = {
    'name' : 'potential',
    'dim' : (1,1),
    'domain' : 'Omega',
    'bases' : {'Omega' : '%s_P1' % geom}
}

variables = {
    'T' : ('unknown field', 'temperature', 0, 'previous'),
    's' : ('test field', 'temperature', 'T'),
    'phi' : ('unknown field', 'potential', 1),
    'psi' : ('test field', 'potential', 'phi'),
    'phi_known' : ('parameter field', 'potential', 'phi'),
}

ics = {
    'ic' : ('Omega', {'T.0' : 0.0}),
}

ebcs = {
    'left' : ('Left', {'T.0' : 0.0, 'phi.0' : 0.0}),
    'right' : ('Right', {'T.0' : 2.0, 'phi.0' : 0.0}),
    'inside' : ('Omega2_Surface', {'phi.0' : 'set_electric_bc'}),
}

def set_electric_bc( bc, ts, coor ):
    y = coor[:,1]
    ymin, ymax = y.min(), y.max()
    val = 2.0 * (((y - ymin) / (ymax - ymin)) - 0.5)
    return val

integral_1 = {
    'name' : 'i1',
    'kind' : 'v',
    'quadrature' : 'gauss_o2_d%d' % dim,
}

equations = {
    '2' : """%.12e * dw_mass_scalar.i1.Omega( s, dT/dt )
             + dw_laplace.i1.Omega( m.thermal_conductivity, s, T )
             = dw_electric_source.i1.Omega( m.electric_conductivity,
                                            s, phi_known ) """ % specific_heat,
    '1' : """dw_laplace.i1.Omega( m.electric_conductivity, psi, phi ) = 0""",
}

fe = {
    'chunk_size' : 100000
}

solver_0 = {
    'name' : 'ls',
    'kind' : 'ls.umfpack',
}

solver_1 = {
    'name' : 'newton',
    'kind' : 'nls.newton',

    'problem' : 'nonlinear',
}

solver_2 = {
    'name' : 'ts',
    'kind' : 'ts.simple',

    't0'    : t0,
    't1'    : t1,
    'dt'    : None,
    'n_step' : n_step, # has precedence over dt!
}

def main():
    from sfepy.base.base import pause, debug, output, default_printer
    from sfepy.base.conf import ProblemConf, get_standard_keywords
    from sfepy.fem import ProblemDefinition
    from sfepy.solvers.generic import solve_evolutionary_op

    default_printer.prefix = 'terel:'
    
    required, other = get_standard_keywords()
    conf = ProblemConf.from_file( __file__, required, other )

    problem = ProblemDefinition.from_conf( conf, init_equations = False )

    # First solve the stationary electric conduciton problem.
    problem.select_variables( ['phi', 'psi'] )
    problem.set_equations( {'eq' : conf.equations['1']} )
    problem.time_update()
    state_el = problem.solve()
    problem.save_state( problem.get_output_name( suffix = 'el' ), state_el )
    
    # Then solve the evolutionary heat conduction problem, using state_el.
    problem.select_variables( ['T', 's', 'phi_known'] )
    problem.set_equations( {'eq' : conf.equations['2']} )
    problem.variables['phi_known'].data_from_data( state_el )
    solve_evolutionary_op( problem )

    output( 'results saved in %s' % problem.get_output_name( suffix = '*' ) )

if __name__ == '__main__':
    main()
